/*{*************************************************************************
  XMBAtom.cpp  -  description
  -------------------
begin                : Fri Jul 21 2006
copyright            : (C) 2006 by ImazInatIon
email                : revanth_nr@students.iiit.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *************************************************************************}*/

#include "XMBAtom.h"
#include <qmessagebox.h>
#include <qstring.h>
#include "xmoleculebuilderdoc.h"

XMBAtom::XMBAtom(QCanvas *canvas, float _atomz, bool lock): QCanvasEllipse( 3, 3, canvas ){
	setBrush( QBrush( QColor(0, 0, 0 ) ) );
	setZ( 2 );
	molecule = new XMBMolecule();
	name = "C";
	symbol = "C";
	locked = lock;
	_atomColor[0]="255";
	_atomColor[1]=_atomColor[2]="0";
	atomZ=_atomz;
	deleted=0;
	canvasSymbol = new QCanvasText( symbol, canvas );
	canvasSymbol->setZ(1);
	canvasSymbol->show();
}

XMBAtom::XMBAtom(QCanvas *canvas, QString atomName, float _atomz, bool lock): QCanvasEllipse( 3, 3, canvas ){
	setBrush( QBrush( QColor(0, 0, 0 ) ) );
	setZ( 2 );
	molecule = new XMBMolecule();
	name = atomName;
	symbol = atomName;
	locked = lock;
	_atomColor[0]="255";
	_atomColor[1]=_atomColor[2]="0";
	atomZ=_atomz;
	deleted=0;
	canvasSymbol = new QCanvasText( symbol, canvas );
	canvasSymbol->setZ(1);
	canvasSymbol->show();
}

XMBAtom::~XMBAtom(){

	XMBBond *edge;

	// inlist
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		inList.remove( edge );
		if( !isAnyhowConnected( edge->getFromAtom() ) ){
			edge->getFromAtom()->setMolecule(new XMBMolecule);
		}
		edge->setToAtom( NULL );
		delete edge;
	}
	// outlist
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		outList.remove( edge );
		if( !isAnyhowConnected( edge->getToAtom() ) ){
			edge->getToAtom()->setMolecule(new XMBMolecule);
		}
		edge->setFromAtom( NULL );
		delete edge;
	}
	if( molecule->removeAtom(this) )
		delete molecule;

	deleted=1;
	canvasSymbol->hide();
	delete canvasSymbol;
}

void XMBAtom::addInBond( XMBBond *bond ) {
	inList.append( bond );
}

void XMBAtom::addOutBond( XMBBond *bond ){
	outList.append( bond );
}

bool XMBAtom::deleteBond( XMBBond *bond){

	bool flag = true;
	XMBBond *edge;

	// inlist
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		if( bond == edge ){
			inList.remove( bond );
			continue;
		}
		flag = false;
	}
	// outlist
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		if( bond == edge ){
			outList.remove( bond );
			continue;
		}
		flag = false;
	}

	return flag;
}

void XMBAtom::transferBonds( XMBAtom *atom){

	XMBBond *edge;

	// if the new bond has an end atom which is already connected to this atom then
	// do not transfer bonds and delete the bond


	// move inlist
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		if( !atom->isConnected( edge->getFromAtom() ) ){
			atom->addInBond( edge );
			edge->setToAtom( atom );
		}
		else
			delete edge;
	}
	inList.clear();
	// move outlist
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		if( !atom->isConnected( edge->getToAtom() ) ){
			atom->addOutBond( edge );
			edge->setFromAtom( atom );
		}
		else
			delete edge;
	}
	outList.clear();
}

#include <math.h>

bool XMBAtom::combine( XMBAtom * a){

	if ( fabsf(a->getAtomZ()-getAtomZ())>1 && a->getMolecule() == getMolecule() )
		return false;

	XMBBond *b;

	// if they are (this)---(a)  then donot proceed it equal to deleting them both
	// REASON - cannnot solve deleting atom with bond to itself problem :( 'ITS RECURSION'

	int flag = 1;
	QPtrListIterator<XMBBond> it1( inList );
	while (( b = it1.current() )) {
		++it1;
		if( b->getFromAtom() != a )
			flag = 0;
	}
	QPtrListIterator<XMBBond> it2( outList );
	while (( b = it2.current() )) {
		++it2;
		if( b->getToAtom() != a )
			flag = 0;
	}
	QPtrListIterator<XMBBond> it4( *(a->getInList()) );
	while (( b = it4.current() )) {
		++it4;
		if( b->getFromAtom() != this )
			flag = 0;
	}
	QPtrListIterator<XMBBond> it5( *(a->getOutList()) );
	while (( b = it5.current() )) {
		++it5;
		if( b->getToAtom() != this )
			flag = 0;
	}

	if( flag )
		return true;

	a->setMolecule( getMolecule() );
	a->setAtomZ( getAtomZ() );
	a->transferBonds( this );
	if( !a->deleted )
		delete a;

	// delete bonds to itself
	QPtrListIterator<XMBBond> it3( inList );
	while (( b = it3.current() )) {
		++it3;
		if( b->getFromAtom() == this )
			delete b;
	}
	return false;
}

void XMBAtom::move(double dx, double dy){
	QCanvasEllipse::move( dx, dy );
	XMBBond *edge;

	// move inlist
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		edge->setToAtom( this );
	}
	// move outlist
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		edge->setFromAtom( this );
	}

	canvasSymbol->move( dx, dy);
}

void XMBAtom::moveBy(double dx, double dy){

	QCanvasEllipse::moveBy( dx, dy );
	XMBBond *edge;

	// move inlist
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		edge->setToAtom( this );
	}
	// move outlist
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		edge->setFromAtom( this );
	}

	canvasSymbol->moveBy( dx, dy);
}

#include <iostream>
using namespace std;
/*
 *  * find angle of inclination of point P1(x1, y1) 
 *   * with respect to point P0(x0, y0)
 *    */
double findinclination( double x0, double y0, double x1, double y1){

	double theta;

	if( x0 == x1 )
		theta = M_PI/2.0;
	else
		theta = atan( (y1-y0)/(x1-x0) );
	if( x1 > x0  && y1 > y0 )
		theta = /*2.0 * M_PI -*/ theta;
	else if( x1 > x0 && y1 < y0 );
//		theta *= -1.0;
	else if( x1 < x0 && y1 > y0 )
		theta += M_PI;// - theta;
	else if( x1 < x0 && y1 < y0 )
		theta -= M_PI;

//	cout<<"("<<x0<<","<<y0<<") ("<<x1<<","<<y1<<") : "<<theta<<endl;
	return theta;
}

void XMBAtom::rotateAbout( double x0, double y0, float phi){

	double r, x1, y1, x2, y2, theta;

	x1 = x();
	y1 = y();

	r = fabs ( sqrt( (x1 - x0) * (x1 - x0) + (y1 - y0) * (y1 - y0) ) );
	theta = findinclination( x0, y0, x1, y1);

	x2 = x0 + r * cos( theta + phi );
	y2 = y0 + r * sin( theta + phi );

//	cout<<symbol<<": phi "<<phi<<"     theta "<<theta<<endl;
	move( x2, y2 );
}

XMBBond  *XMBAtom::isConnected( XMBAtom *atom){

	XMBBond *bond;

	QPtrListIterator<XMBBond> it1( inList );
	while( bond = it1.current() ){
		++it1;
		if( bond->fromAtom == atom )
			return bond;
	}
	QPtrListIterator<XMBBond> it2( outList );
	while( bond = it2.current() ){
		++it2;
		if( bond->toAtom == atom )
			return bond;
	}

	return NULL;
}

bool XMBAtom::isAnyhowConnected( XMBAtom *atom){

	if( atom == NULL )
		return false;
	if( isInListConnected( atom ) )
		return true;
	if( isOutListConnected( atom ) )
		return true;
	return false;
}

bool XMBAtom::isInListConnected( XMBAtom *atom){

	if( atom == NULL )
		return false;
	if( flag == 1 )
		return false;
	flag = 1;

	XMBBond *bond;

	QPtrListIterator<XMBBond> it1( inList );
	while( bond = it1.current() ){
		++it1;
		if( bond->fromAtom == atom ){
			flag = 0;
			return true;
		}
		if(bond->fromAtom &&  bond->fromAtom->isInListConnected( atom ) ){
			flag = 0;
			return true;
		}
	}

	flag = 0;
	return false;
}

bool XMBAtom::isOutListConnected( XMBAtom *atom){

	if( atom == NULL )
		return false;
	if( flag == 1)
		return false;
	flag = 1;

	XMBBond *bond;

	QPtrListIterator<XMBBond> it2( outList );
	while( bond = it2.current() ){
		++it2;
		if( bond->toAtom == atom ){
			flag = 0;
			return true;
		}
		if( bond->toAtom && bond->toAtom->isOutListConnected( atom ) ){
			flag = 0;
			return true;
		}
	}

	flag = 0;
	return false;
}

void XMBAtom::setMolecule( XMBMolecule *m){

	if( molecule == m )
		return;

	if( molecule->removeAtom(this) )
		delete molecule;

	molecule = m;
	molecule->addAtom(this);
	XMBBond *edge;

	// move inlist
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		edge->getFromAtom()->setMolecule( m);
	}
	// move outlist
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		edge->getToAtom()->setMolecule( m);
	}

}

XMBMolecule *XMBAtom::getMolecule( void ){
	return molecule;
}

int XMBAtom::getBondCount( void ){

	return inList.count() + outList.count();
}

QPtrList<XMBBond> *XMBAtom::getInList(){
	return &inList;
}

QPtrList<XMBBond> *XMBAtom::getOutList(){
	return &outList;
}

XMBAtom *XMBAtom::getFirstConnectedAtom(){

	if( inList.count() )
		return inList.first()->getFromAtom();
	if( outList.count() )
		return outList.first()->getToAtom();
	return NULL;
}

void XMBAtom::writeMoleculeToFile(QTextStream *s, QValueList<QPair<XMBAtom*,XMBAtom*> > *l){

	(*s)<<"ATOM "<<x()<<" "<<y()<<endl;

	XMBBond *edge;
	QPtrListIterator<XMBBond> it1( inList );
	while (( edge = it1.current() )) {
		++it1;
		if( l->find( qMakePair( edge->getFromAtom(), this ) ) == l->end() && l->find( qMakePair( this, edge->getFromAtom()) ) == l->end() ){
			(*s)<<"BOND_IN "<<edge->bondCount<<endl;
			l->append( qMakePair(edge->getFromAtom(), this) );
			edge->getFromAtom()->writeMoleculeToFile( s, l);
		}
	}
	QPtrListIterator<XMBBond> it2( outList );
	while (( edge = it2.current() )) {
		++it2;
		if( l->find( qMakePair( edge->getToAtom(), this ) ) == l->end() && l->find( qMakePair( this, edge->getToAtom()) ) == l->end() ){
			(*s)<<"BOND_OUT "<<edge->bondCount<<endl;
			l->append( qMakePair(edge->getToAtom(), this) );
			edge->getToAtom()->writeMoleculeToFile( s, l);
		}
	}

	(*s)<<"END"<<endl;
}

void XMBAtom::readMoleculeFromFile(QTextStream *s, XMBMolecule *molecule, XMoleculeBuilderDoc *doc){

	XMBAtom *atom;
	XMBBond *bond;
	int x, y, count;
	QString str;
	XMBBondType bondType;

	(*s)>>str;
	(*s)>>x>>y;
	move( x, y);
	setMolecule( molecule );

	while( 1 ){
		(*s)>>str;
		if( str == "END" )
			return;
		if( str == "BOND_IN" ){
			(*s)>>count;
			atom = new XMBAtom( doc->canvas , 0);
			atom->readMoleculeFromFile( s , molecule, doc);
			atom->show();
			bond = new XMBBond( atom, this, doc);
			switch( count ){
				case 3:
					bond->addBond( bondType );
				case 2:
					bond->addBond( bondType );
				case 1:
					bond->addBond( bondType );
			}
			continue;
		}
		if( str == "BOND_OUT" ){
			(*s)>>count;
			atom = new XMBAtom( doc->canvas , 0);
			atom->readMoleculeFromFile( s , molecule, doc);
			atom->show();
			bond = new XMBBond( this, atom, doc);
			switch( count ){
				case 3:
					bond->addBond( bondType );
				case 2:
					bond->addBond( bondType );
				case 1:
					bond->addBond( bondType );
			}
			continue;
		}
		break;
	}

}

void XMBAtom::startHover(void){
	setBrush( QBrush( QColor( 0, 255, 0 ) ) ) ;
	setSize( 6, 6 );
}

void XMBAtom::stopHover(void){
	setBrush( QBrush( QColor( 0, 0, 0 ) ) ) ;
	setSize( 3, 3 );
}

QString XMBAtom::getName( void ) {
	return name;
}

QString XMBAtom::getSymbol( void ) {
	return symbol;
}

void XMBAtom::setName( QString n){
	name = n;

}

void XMBAtom::setSymbol( QString s){
	symbol = s;
	canvasSymbol->setText(symbol);
}

float XMBAtom::getAtomZ( void ) {
	return atomZ;
}

void XMBAtom::setAtomZ( float z ) {
	
	if( atomZ == z )
		return;

	atomZ=z;
	
	QPtrListIterator<XMBBond> it( outList );
	while ( it ) {
		((XMBBond*)(*it))->setBond();
		++it;
	}
}
