/*{**************************************************************************
                  XMBBond.cpp  -  description
                       -------------------
     
     begin                : Fri Jul 21 2006
     copyright            : (C) 2006 by ImazInatIon
     email                : revanth_nr@students.iiit.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *************************************************************************}*/

#include "XMBBond.h"
#include "XMBAtom.h"
#include "XMBBondLine.h"
#include "XMBStereoBond.h"
#include <qmessagebox.h>
#include <math.h>

#include <iostream>
using namespace std;
// constructor given end atoms
XMBBond::XMBBond( XMBAtom* from, XMBAtom *to, XMoleculeBuilderDoc* pDoc){

	fromAtom = from;
	toAtom = to;
	from->addOutBond( this );
	to->addInBond( this );
	doc = pDoc;
	bondCount = 0;
	bondlength=sqrt((toAtom->x()-fromAtom->x())*(toAtom->x()-fromAtom->x()) + (toAtom->y()-fromAtom->y())*(toAtom->y()-fromAtom->y()));
}

// destructor
XMBBond::~XMBBond(){

	int flag = 0;
	switch( bondCount-1 ){
		case 2:
			bondCount = -1;
			delete bond3;
		case 1:
			bondCount = -1;
			delete bond2;
		case 0:
			bondCount = -1;
			if( bond1Type == XMBBT_StereoDown || bond1Type == XMBBT_StereoUp)
				delete  stereoBond;
			else
				delete bond1;
	}
	if( toAtom && toAtom->deleteBond( this ) ){
//		flag = 1;
//		delete toAtom;
	}
	if( fromAtom && fromAtom->deleteBond( this ) ){
//		flag = 1;
//		delete fromAtom;
	}
	if( flag==0 && toAtom && fromAtom && !toAtom->isAnyhowConnected( fromAtom ) )
		toAtom->setMolecule( new XMBMolecule() );
}

// set bond
void XMBBond::setBond(void) {

	if( fromAtom == NULL || toAtom == NULL )
		return;

	double x1, y1_, x2, y2, x11, x12, y11, y12, x21, x22, y21, y22, d, k;
	double d2 = 30;

	bondlength=sqrt((toAtom->x()-fromAtom->x())*(toAtom->x()-fromAtom->x()) + (toAtom->y()-fromAtom->y())*(toAtom->y()-fromAtom->y()));

	x1 = fromAtom->x();
	y1_ = fromAtom->y();
	x2 = toAtom->x();
	y2 = toAtom->y();

	// handle exception x1==x2 or y1==y2
	if( x1 == x2 ){
		x11 = x21 = x1 + 6;
		x12 = x22 = x1 - 6;
		y11 = y12 = y1_;
		y21 = y22 = y2;
	}
	else if( y1_ == y2 ){
		x11 = x12 = x1;
		x21 = x22 = x2;

		y11 = y21 = y1_ + 6;
		y12 = y22 = y1_ - 6;
	}
	else{

		k = ( y2 - y1_ ) / ( x2 - x1 );

		x11 = x1 + sqrt( d2*k*k / ( k*k + 1 ) );
		y11 = y1_ - sqrt( d2*k*k / ( k*k + 1 ) ) / k;
		x12 = x1 - sqrt( d2*k*k / ( k*k + 1 ) );
		y12 = y1_ + sqrt( d2*k*k / ( k*k + 1 ) ) / k;

		x21 = x2 + sqrt( d2*k*k / ( k*k + 1 ) );
		y21 = y2 - sqrt( d2*k*k / ( k*k + 1 ) ) / k;
		x22 = x2 - sqrt( d2*k*k / ( k*k + 1 ) );
		y22 = y2 + sqrt( d2*k*k / ( k*k + 1 ) ) / k;
	}

	switch( bondCount-1 ){
		case 2:
			bond3->setFromPoint( x11, y11 );
			bond3->setToPoint( x21, y21 );
		case 1:
			bond2->setFromPoint( x12, y12);
			bond2->setToPoint( x22,  y22 );
		case 0:
			if( bond1Type == XMBBT_StereoDown || bond1Type == XMBBT_StereoUp){
				stereoBond->setFromPoint( x1, y1_ );
				stereoBond->setToPoint( x21, y21, x22, y22 );
				if(!toAtom->locked)
					toAtom->setAtomZ(sqrt((x2-x1)*(x2-x1) + (y2-y1_)*(y2-y1_))*tan(stereoBond->getAngle()*M_PI/180.0) + fromAtom->getAtomZ() );
			}
			else{
				bond1->setFromPoint( x1, y1_);
				bond1->setToPoint( x2, y2 );
				if(!toAtom->locked)
					toAtom->setAtomZ(fromAtom->getAtomZ());
			}
			break;
	}
}

// set Bond Length
void XMBBond::setLength( QString length )
{
	float ratio=atof(length)/bondlength;
	toAtom->moveBy((1-ratio)*(fromAtom->x()-toAtom->x()),(1-ratio)*(fromAtom->y()-toAtom->y()));
	bondlength=atof(length);
}

//get BondLength
QString XMBBond::getLength(void)
{
	char* length=(char*)malloc(8*sizeof(char));
	sprintf(length,"%f",bondlength);
	return QString(length);
}

// change from Atom
void XMBBond::setFromAtom( XMBAtom* from){

	fromAtom = from;
	if( fromAtom != NULL )
		setBond();
}

// change to Atom
void XMBBond::setToAtom( XMBAtom* to){

	toAtom = to;
	if( toAtom != NULL )
		setBond();
}

// get from atom
XMBAtom *XMBBond::getFromAtom(){
	return fromAtom;
}

// get to atom
XMBAtom *XMBBond::getToAtom(){
	return toAtom;
}

//get molecule
XMBMolecule *XMBBond::getMolecule( void ){
	return (getFromAtom()->getMolecule());
}

// add bond
void XMBBond::addBond( XMBBondType bondType , float angle ){

	double x1, y1_, x2, y2, x11, x12, y11, y12, x21, x22, y21, y22, d, k;
	double d2 = 30;

	// cannot add stereo bond as second bond
	if( bondType == XMBBT_StereoDown || bondType == XMBBT_StereoUp )
		if( bondCount != 0 )
			return;
	// cannot add any bond if stereo bond is present
	if( bondCount == 1 && (bond1Type == XMBBT_StereoUp || bond1Type == XMBBT_StereoDown) )
		return;

	if(bondCount == 3)
		return;
	
	x1 = fromAtom->x();
	y1_ = fromAtom->y();
	x2 = toAtom->x();
	y2 = toAtom->y();

	// handle exception x1=x2 or y1==y2
	if( x1 == x2 ){
		x11 = x21 = x1 + 6;
		x12 = x22 = x1 - 6;
		y11 = y12 = y1_;
		y21 = y22 = y2;
	}
	else if( y1_ == y2 ){
		x11 = x12 = x1;
		x21 = x22 = x2;

		y11 = y21 = y1_ + 6;
		y12 = y22 = y1_ - 6;
	}
	else{

		k = ( y2 - y1_ ) / ( x2 - x1 );

		x11 = x1 + sqrt( d2*k*k / ( k*k + 1 ) );
		y11 = y1_ - sqrt( d2*k*k / ( k*k + 1 ) ) / k;
		x12 = x1 - sqrt( d2*k*k / ( k*k + 1 ) );
		y12 = y1_ + sqrt( d2*k*k / ( k*k + 1 ) ) / k;

		x21 = x2 + sqrt( d2*k*k / ( k*k + 1 ) );
		y21 = y2 - sqrt( d2*k*k / ( k*k + 1 ) ) / k;
		x22 = x2 - sqrt( d2*k*k / ( k*k + 1 ) );
		y22 = y2 + sqrt( d2*k*k / ( k*k + 1 ) ) / k;
	}

	if( bondType == XMBBT_SingleBond || bondType == XMBBT_DashedLine )
		switch( bondCount ){
			case 2:
				bond3 = new XMBBondLine( QPoint( x11, y11), QPoint( x21, y21), doc, this, bondType == XMBBT_DashedLine);
				bond3Type = bondType;
				break;
			case 1:
				bond2 = new XMBBondLine( QPoint( x12, y12), QPoint( x22, y22), doc, this, bondType ==XMBBT_DashedLine);
				bond2Type = bondType;
				break;
			case 0:
				bond1 = new XMBBondLine( QPoint( x1, y1_), QPoint( x2,y2), doc, this, bondType == XMBBT_DashedLine);
				bond1Type = bondType;
				break;
		}
	if( bondType == XMBBT_StereoUp){
		stereoBond = new XMBStereoBond( QPoint( x1, y1_), QPoint( x21,y21), QPoint(x22,y22), doc, this, bondType, angle);
		bond1Type = bondType;
	}
	if( bondType == XMBBT_StereoDown){
		stereoBond = new XMBStereoBond( QPoint( x1, y1_), QPoint( x21,y21), QPoint(x22,y22), doc, this, bondType, angle);
		bond1Type = bondType;
	}

	bondCount++;
}

// move bond
void XMBBond::moveBy( double dx, double dy){
	
	fromAtom->moveBy(dx, dy);
	toAtom->moveBy(dx, dy);
}

// delete bond
bool XMBBond::deleteBond( XMBStereoBond *bond ){
//	if(bondCount==1)
		return true;
//	return false;
}

bool XMBBond::deleteBond( XMBBondLine *bond ){

	if( bondCount < 0 )
		return false;

	switch( bondCount-1 ){
		case 2:
			bondCount--;
			if( bond2 == bond )
				bond2 = bond3;
			if( bond1 == bond )
				bond1 = bond3;
			bond3 = NULL;
			//delete bond3;
			break;
		case 1:
			bondCount--;
			if( bond1 == bond )
				bond1 = bond2;
			bond2 = NULL;
			//delete bond2;
			break;
		case 0:
			bondCount--;
			//delete bond1;

			// bond shud be deleted now
			return true;
	}
	setBond();
	return false;
}

// change colour
void XMBBond::setColor( int r, int g, int b){

}
