/*{*************************************************************************
				  XMBStereoBond.cpp  -  description
					  -------------------
    begin                : Fri Jul 21 2006
    copyright            : (C) 2006 by ImazInatIon
    email                : revanth_nr@students.iiit.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *************************************************************************}*/

#include "XMBStereoBond.h"
#include "XMBBond.h"
#include <cmath>
#include <qmessagebox.h>

// stereo down pattern can be Qt::DiagCrossPattern or Qt::VerPattern

class XMBBond;

XMBStereoBond::XMBStereoBond( QPoint from1, QPoint from2,  QPoint to, XMoleculeBuilderDoc *pDoc, XMBBond *b, XMBBondType _bondType, float angle): QCanvasPolygon( pDoc->canvas ){
	setColor( 0, 0, 255);
//	canvasSymbol = new QCanvasText( "a", pDoc->canvas );
//	canvasSymbol->setZ(1);
//	canvasSymbol->show();
	bondType = _bondType;
	if( bondType == XMBBT_StereoDown )
	{
		setBrush( QBrush( QColor(color[0],color[1], color[2]) , Qt::VerPattern));
//		pangle = -angle;
	}
	else if( bondType == XMBBT_StereoUp )
	{
		setBrush( QBrush( QColor(color[0],color[1], color[2]) , Qt::SolidPattern));
//		pangle = angle;
	}
	pangle = angle;
	bond = b;
	pa = new QPointArray(3);
	pa->putPoints(0,3, from1.x(), from1.y(), from2.x(), from2.y(), to.x(), to.y() );
	setPoints(*pa);
	setZ( 1 );
	show();
}

XMBStereoBond::~XMBStereoBond(){
	
	if( bond->deleteBond(this) )
	{
//		delete bond;
	}
}


void XMBStereoBond::setFromPoint( double x, double y){
	pa->putPoints(0,1, x, y);
	setPoints(*pa);
}

void XMBStereoBond::setToPoint( double x1, double y1, double x2, double y2 ){
	pa->setPoint(1, x1, y1);
	pa->setPoint(2, x2, y2);
	setPoints(*pa);
}

QPointArray *XMBStereoBond::getPointsArray( void ){
	return pa;
}

XMBBond *XMBStereoBond::getBond(void){
	return bond;
}

void XMBStereoBond::deleteBond(void) {
	delete bond;	
}

void XMBStereoBond::setAngle(float angle) {
	if(angle<0)
		pangle=0;
	else if(angle<90)
		pangle=angle;
	else
		pangle=89.5;
	int x1=pa->point(0).x();
	int y1_=pa->point(0).y();
	int x2=(pa->point(1).x()+pa->point(2).x())/2;
	int y2=(pa->point(1).y()+pa->point(2).y())/2;
	float length=sqrt((x2-x1)*(x2-x1)+(y2-y1_)*(y2-y1_));
	bond->toAtom->setAtomZ( bond->fromAtom->getAtomZ() + length*tan(pangle)*(bondType==2?1:-1));
}

float XMBStereoBond::getAngle(void) {
	return pangle;
}

void XMBStereoBond::moveBy( double dx, double dy){

	bond->moveBy( dx, dy );
}

void XMBStereoBond::myMoveBy( double dx, double dy ){

	QCanvasPolygon::moveBy( dx, dy );
}

void XMBStereoBond::setColor( int r, int g, int b){
	color[0] = r;
	color[1] = g;
	color[2] = b;
}

void XMBStereoBond::startHover(void){
	if( bondType == XMBBT_StereoDown )
		setBrush( QBrush( QColor(0,255,0) , Qt::VerPattern));
	else if( bondType == XMBBT_StereoUp )
		setBrush( QBrush( QColor(0,255,0) , Qt::SolidPattern));
}

void XMBStereoBond::stopHover(void){
	if( bondType == XMBBT_StereoDown )
		setBrush( QBrush( QColor(color[0],color[1], color[2]) , Qt::VerPattern));
	else if( bondType == XMBBT_StereoUp )
		setBrush( QBrush( QColor(color[0],color[1], color[2]) , Qt::SolidPattern));
}

