/*{*************************************************************************
                          xmoleculebuilder.h  -  description
                             -------------------
    begin                : Fri Jul 21 2006
    copyright            : (C) 2006 by ImazInatIon
    email                : revanth_nr@students.iiit.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *************************************************************************}*/

#ifndef XMOLECULEBUILDER_H
#define XMOLECULEBUILDER_H

// include files for QT
#include <qapp.h>
#include <qmainwindow.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qstring.h>
#include <qpixmap.h>
#include <qmsgbox.h>
#include <qfiledialog.h>
#include <qprinter.h>
#include <qpainter.h>
//#include <qbytearray.h>


#include "XMBMolecule.h"

// application specific includes
#include "xmoleculebuilderview.h"
#include "xmoleculebuilderdoc.h"
#include "XMBI3dView.h"


class XMoleculeBuilderView;
class XMoleculeBuilderDoc;
class XMBI3dView;

/**
 * This Class is the base class for your application. It sets up the main
 * window and providing a menubar, toolbar
 * and statusbar. For the main view, an instance of class XMoleculeBuilderView is
 * created which creates your view.
 */
class XMoleculeBuilderApp : public QMainWindow
{
	Q_OBJECT

	public:
		/** construtor */
		XMoleculeBuilderApp();
		/** destructor */
		~XMoleculeBuilderApp();
		/** initializes all QActions of the application */
		void initActions();
		/** initMenuBar creates the menu_bar and inserts the menuitems */
		void initMenuBar();
		/** this creates the toolbars. Change the toobar look and add new toolbars in this
		 * function */
		void initToolBar();
		/** setup the statusbar */
		void initStatusBar();
		/** setup the document*/
		void initDoc();
		/** setup the mainview*/
		void initView();
		/** setup the 3dviewer*/
		void initI3dView();

		/** overloaded for Message box on last window exit */
		bool queryExit();

	public slots:

		/** create new document */
		void slotFileNew();
		/** open a document */
		void slotFileOpen();
		/** close the actual file */
		void slotFileClose();
		/** save a document */
		void slotFileSave();
		/** save a document under a different filename*/
		void slotFileSaveAs();
		/** print the actual file */
		void slotFilePrint();
		/** exits the application */
		void slotFileQuit();
		/** toggle the toolbar*/
		void slotViewToolBar(bool toggle);
		/** toggle the statusbar*/
		void slotViewStatusBar(bool toggle);
		/** toggle the librarybar*/
		void slotViewLibraryBar(bool toggle);
		
		void slotInsertMolecule(void);
		void slotImportMolecule(void);

		/** shows an about dlg*/
		void slotHelpAbout();

		// XMB specific
		void slotSelectAddBond(void );
		void slotSelectViewMode(void );
		void slotSelectRotateMode(void );
		void slotSelectDeleteItem(void );
		void slotSelectAddDashedBond(void );
		void slotDisplayBondDetail(char* );
		void slotSelectAddStereoUp(void );
		void slotSelectAddStereoDown(void );
		void slotSelectFixedLength(void );
		void slotSelectFixedAngle(void );
		void slotZigzagStructure(void );
		void slotZigzagAtomCount(int );
		void slotSelectI3DView(void);

		// libraby
		void slotAddBenzene(void);
		void slotAddCubane(void);
		void slotAddHexane(void);
		void slotAddMegaphon(void);
		void slotAddProp(void);
		void slotAddWater(void);

	private:

		/** view is the main widget which represents your working area. The View
		 * class should handle all events of the view widget.  It is kept empty so
		 * you can create your view according to your application's needs by
		 * changing the view class.
		 */
		XMoleculeBuilderView *view;
		/** doc represents your actual document and is created only once. It keeps
		 * information such as filename and does the serialization of your files.
		 */
		XMoleculeBuilderDoc *doc;
		/** i3dviewer is the opengl viewer.
		  */
		XMBI3dView *i3dviewer;

		/** file_menu contains all items of the menubar entry "File" */
		QPopupMenu *pFileMenu;
		/** view_menu contains all items of the menubar entry "View" */
		QPopupMenu *pViewMenu;
		/** insert_menu contains all items of the menubar entry Insert" */
		QPopupMenu *pInsertMenu;
		/** view_menu contains all items of the menubar entry "Help" */
		QPopupMenu *pHelpMenu;
		/** the main toolbar */
		QToolBar *fileToolbar;
		QToolBar *libraryToolbar;
		/** actions for the application initialized in initActions() and used to en/disable them
		 * according to your needs during the program */
		QAction *fileNew;
		QAction *fileOpen;
		QAction *fileSave;
		QAction *fileSaveAs;
		QAction *fileClose;
		QAction *filePrint;
		QAction *fileQuit;
		QAction *insertMolecule;
		QAction *importMolecule;


		QAction *viewToolBar;
		QAction *viewStatusBar;
		QAction *viewLibraryBar;

		QAction *helpAboutApp;

		// XMB specific controls
		QToolBar *drawToolBar;
		QToolBar *libraryToolBar;
		QActionGroup *viewGrp;

		// for drawToolBar
		QAction *drawBond;
		QAction *drawDashedBond;
		QAction *drawStereoUp;
		QAction *drawStereoDown;
		QAction *deleteItem;
		QAction *viewMode;
		QAction *rotateMode;
		QAction *fixedLength;
		QAction *fixedAngle;
		QAction *zigzagStructure;

		//for I3D view ToolBar
		QAction *I3DView;

		// for libraryToolBar
		QAction *xmtBenzene;
		QAction *xmtCubane;
		QAction *xmtHexane;
		QAction *xmtMegaphon;
		QAction *xmtProp;
		QAction *xmtWater;

};


//XMoleculeBuilderApp *xmoleculebuilder;//=new XMoleculeBuilderApp();
#endif

